# Generated by Django 2.1.7 on 2019-10-01 14:36

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("dynamic_preferences", "0005_auto_20181120_0848"),
    ]

    operations = [
        migrations.AlterField(
            model_name="globalpreferencemodel",
            name="section",
            field=models.CharField(
                blank=True,
                db_index=True,
                default=None,
                max_length=150,
                null=True,
                verbose_name="Section Name",
            ),
        ),
    ]
